package com.example.demo.java;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.department.model.DepartmentEntity;
import com.example.demo.entrysData.model.EntrysDataEntity;
import com.example.demo.entrysData.service.EntrysDataService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class EntrysDataTests {
    @Autowired
    private EntrysDataService entrysDataService;

    private EntrysDataEntity entrysData;

    @BeforeEach
    void createData() {
        removeData();

        entrysData = entrysDataService
                .create(new EntrysDataEntity("login1", "pass1", "user", new DepartmentEntity("ГУМ")));
        entrysDataService.create(
                new EntrysDataEntity("login2", "pass2", "user", new DepartmentEntity("IST")));
        entrysDataService.create(
                new EntrysDataEntity("login3", "pass3", "user", new DepartmentEntity("IYA")));
    }

    @AfterEach
    void removeData() {
        entrysDataService.getAll(0L).forEach(item -> entrysDataService.delete(item.getId()));
    }

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> entrysDataService.get(0L));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(3, entrysDataService.getAll(0L).size());
        Assertions.assertEquals(entrysData, entrysDataService.get(entrysData.getId()));
    }

    @Test
    void createNotUniqueTest() {
        final EntrysDataEntity nonUniquedirection = new EntrysDataEntity("login4", "pass4", "user",
                new DepartmentEntity("IYA"));
        Assertions.assertThrows(IllegalArgumentException.class, () -> entrysDataService.create(nonUniquedirection));
    }

    @Test
    void createNullableTest() {
        final EntrysDataEntity nullabledirection = new EntrysDataEntity(null, null, null, null);
        Assertions.assertThrows(
                DataIntegrityViolationException.class,
                () -> entrysDataService.create(nullabledirection));
    }

    @Test
    void updateTest() {
        final String testLog = "TESTLOG";
        final String testPas = "TESTPAS";
        final DepartmentEntity testDep = new DepartmentEntity("TESTDEP");
        final EntrysDataEntity entity = entrysDataService.get(3L);
        final String oldLog = entity.getLogin();
        final String oldPas = entity.getLogin();
        final DepartmentEntity oldDep = entity.getDepartment();
        final EntrysDataEntity newEntity = entrysDataService.update(3L,
                new EntrysDataEntity(testLog, testPas, entity.getRole(), testDep));
        Assertions.assertEquals(3, entrysDataService.getAll(0L).size());
        Assertions.assertEquals(newEntity, entrysDataService.get(3L));

        Assertions.assertEquals(testLog, newEntity.getLogin());
        Assertions.assertNotEquals(oldLog, newEntity.getLogin());

        Assertions.assertEquals(testPas, newEntity.getPassword());
        Assertions.assertNotEquals(oldPas, newEntity.getPassword());

        Assertions.assertEquals(testDep, newEntity.getDepartment());
        Assertions.assertNotEquals(oldDep, newEntity.getDepartment());
    }

    @Test
    void deleteTest() {
        entrysDataService.delete(3L);
        Assertions.assertEquals(2, entrysDataService.getAll(0L).size());
        final EntrysDataEntity last = entrysDataService.get(2L);
        Assertions.assertEquals(2L, last.getId());

        final EntrysDataEntity newEntity = entrysDataService
                .create(new EntrysDataEntity("user4", "12345", "user",
                        new DepartmentEntity("testtt")));
        Assertions.assertEquals(3, entrysDataService.getAll(0L).size());
        Assertions.assertEquals(4L, newEntity.getId());
    }

    @Test
    void updatePasswordTest() {
        final String newPas = "TESTPAS";
        final EntrysDataEntity entity = entrysDataService.get(1L);
        final String oldPas = entity.getPassword();
        final EntrysDataEntity newEntity = entrysDataService.updatePassword(1L, newPas, oldPas);

        Assertions.assertEquals(newPas, newEntity.getPassword());
        Assertions.assertNotEquals(oldPas, newEntity.getPassword());
    }
}
